/***************************************************************************
 *   Copyright (C) 2004 by Hans Oischinger                                 *
 *   hans.oischinger@kdemail.net                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Steet, Fifth Floor, Boston, MA 02110-1301, USA.             *
 ***************************************************************************/
#ifndef TOOLMANAGER_H
#define TOOLMANAGER_H

#include <qobject.h>
#include <qdict.h>

#include "generictool.h"

class SelectTool;
class VectorFactoryTool;
class VectorElement;
class SelectNodesTool;
/**
@author Hans Oischinger
*/
class ToolManager : public QObject
{
  Q_OBJECT
public:
  ToolManager();
  ~ToolManager();

  void registerTool( GenericTool *tool );
  void unregisterTool( GenericTool *tool );

  void setDefaultTool();
  void setActiveTool( GenericTool * );
  GenericTool *activeTool() const { return m_activeTool; }

  bool mouseEvent( QMouseEvent* event, const QPoint& );
  bool keyEvent( QEvent* event );

  const QDict<GenericTool> &tools() { return m_tools; }

  //SelectTool* getToolSelect() { return m_selectTool; }

signals:
  void activeToolChanged();

protected:
  void init();
  VectorElement* createPrototype( unsigned int type );

  GenericTool *m_activeTool;
  QDict<GenericTool> m_tools;
  SelectTool *m_selectTool;
  VectorFactoryTool *m_vecRectTool;
  VectorFactoryTool *m_vecEllipseTool;
  SelectNodesTool *m_selectNodesTool;
};

#endif
