# - Try to find precompiled headers support for GCC 3.4 and 4.x
# Once done this will define:
#
# Variable:
#   PCHSupport_FOUND
#
# Macro:
#   ADD_PRECOMPILED_HEADER

IF(CMAKE_COMPILER_IS_GNUCXX)
    EXEC_PROGRAM(${CMAKE_CXX_COMPILER} 
        ARGS --version OUTPUT_VARIABLE _compiler_output)
    STRING(REGEX REPLACE ".* ([0-9]\\.[0-9]\\.[0-9]) .*" "\\1" 
           gcc_compiler_version ${_compiler_output})
    IF(gcc_compiler_version MATCHES "4\\.[0-9]\\.[0-9]")
        SET(PCHSupport_FOUND TRUE)
    ELSE(gcc_compiler_version MATCHES "4\\.[0-9]\\.[0-9]")
        IF(gcc_compiler_version MATCHES "3\\.4\\.[0-9]")
            SET(PCHSupport_FOUND TRUE)
        ENDIF(gcc_compiler_version MATCHES "3\\.4\\.[0-9]")
    ENDIF(gcc_compiler_version MATCHES "4\\.[0-9]\\.[0-9]")
ENDIF(CMAKE_COMPILER_IS_GNUCXX)


MACRO(_PCH_GET_COMPILE_FLAGS _out_compile_flags)
	STRING(TOUPPER "CMAKE_CXX_FLAGS_${CMAKE_BUILD_TYPE}" _flags_var_name)
	SET(${_out_compile_flags} ${${_flags_var_name}} )
	
	GET_TARGET_PROPERTY(_targetType ${_PCH_current_target} TYPE)
	
	IF (${_targetType} STREQUAL SHARED_LIBRARY)
		LIST(APPEND ${_out_compile_flags} "${${_out_compile_flags}} -fPIC")
	ENDIF (${_targetType} STREQUAL SHARED_LIBRARY)
	
	GET_DIRECTORY_PROPERTY(DIRINC INCLUDE_DIRECTORIES )
	
	FOREACH(item ${DIRINC})
		LIST(APPEND ${_out_compile_flags} "${_PCH_include_prefix}${item}")
	ENDFOREACH(item)
	
	GET_DIRECTORY_PROPERTY(_directory_flags DEFINITIONS)
	
	LIST(APPEND ${_out_compile_flags} ${_directory_flags})
	LIST(APPEND ${_out_compile_flags} ${CMAKE_CXX_FLAGS} )
	
	SEPARATE_ARGUMENTS(${_out_compile_flags})
	
ENDMACRO(_PCH_GET_COMPILE_FLAGS)


MACRO(_PCH_WRITE_PCHDEP_CXX _targetName _include_file)
	FILE (WRITE ${CMAKE_CURRENT_BINARY_DIR}/${_targetName}_pchdep.cxx 
		"#include \"${_include_file}\" 
		int testfunction()
		{
			return 0;
		}" ) 
ENDMACRO(_PCH_WRITE_PCHDEP_CXX )

MACRO(_PCH_GET_COMPILE_COMMAND out_command _input _output)
	FILE(TO_NATIVE_PATH ${_input} _native_input)
	FILE(TO_NATIVE_PATH ${_output} _native_output)
	
	IF(CMAKE_COMPILER_IS_GNUCXX)
		IF (CMAKE_CXX_COMPILER_ARG1)    
			STRING(REGEX REPLACE "^ +" "" pchsupport_compiler_cxx_arg1 ${CMAKE_CXX_COMPILER_ARG1})
			SET(${out_command} ${CMAKE_CXX_COMPILER} ${pchsupport_compiler_cxx_arg1} ${_compile_FLAGS}   -x c++-header -o ${_output} ${_input} )
		ELSE (CMAKE_CXX_COMPILER_ARG1)
			SET(${out_command} ${CMAKE_CXX_COMPILER}  ${_compile_FLAGS}  -x c++-header -o ${_output} ${_input} )
		ENDIF(CMAKE_CXX_COMPILER_ARG1)
	ELSE(CMAKE_COMPILER_IS_GNUCXX)            
		SET(_dummy_str "#include <${_input}>")
		FILE(WRITE ${CMAKE_CURRENT_BINARY_DIR}/pch_dummy.cpp ${_dummy_str})
		SET(${out_command} ${CMAKE_CXX_COMPILER} ${_compile_FLAGS} /c /Fp${_native_output} /Yc${_native_input} pch_dummy.cpp )
	ENDIF(CMAKE_COMPILER_IS_GNUCXX)    
ENDMACRO(_PCH_GET_COMPILE_COMMAND )

MACRO(_PCH_GET_TARGET_COMPILE_FLAGS _cflags  _header_name _pch_path  )
	FILE(TO_NATIVE_PATH ${_pch_path} _native_pch_path)
	message(${_native_pch_path})
	
	IF(CMAKE_COMPILER_IS_GNUCXX)    
		set(${_cflags} "-include ${CMAKE_CURRENT_BINARY_DIR}/${_header_name} -Winvalid-pch" )
	ELSE(CMAKE_COMPILER_IS_GNUCXX)
		set(${_cflags} "/Fp${_native_pch_path} /Yu${_header_name}" )    
	ENDIF(CMAKE_COMPILER_IS_GNUCXX)
ENDMACRO(_PCH_GET_TARGET_COMPILE_FLAGS )

MACRO(ADD_PRECOMPILED_HEADER _targetName _input )
	SET(_PCH_current_target ${_targetName})
	
	IF(NOT CMAKE_BUILD_TYPE)
		MESSAGE(FATAL_ERROR  "This is the ADD_PRECOMPILED_HEADER macro. " "You must set CMAKE_BUILD_TYPE!" )
	ENDIF(NOT CMAKE_BUILD_TYPE)
	
	GET_FILENAME_COMPONENT(_name ${_input} NAME)
	GET_FILENAME_COMPONENT(_path ${_input} PATH)
	SET(_outdir "${CMAKE_CURRENT_BINARY_DIR}/${_name}.gch")
	SET(_output "${_outdir}/${_targetName}_${CMAKE_BUILD_TYPE}.h++")
	
	GET_TARGET_PROPERTY(_targetType ${_PCH_current_target} TYPE)
	_PCH_WRITE_PCHDEP_CXX(${_targetName} ${_input} )
	
	IF (${_targetType} STREQUAL SHARED_LIBRARY)
		ADD_LIBRARY(${_targetName}_pchdep SHARED ${CMAKE_CURRENT_BINARY_DIR}/${_targetName}_pchdep.cxx)
	ELSE (${_targetType} STREQUAL SHARED_LIBRARY)
		ADD_LIBRARY(${_targetName}_pchdep STATIC ${CMAKE_CURRENT_BINARY_DIR}/${_targetName}_pchdep.cxx)
	ENDIF (${_targetType} STREQUAL SHARED_LIBRARY)
	
	ADD_CUSTOM_COMMAND(OUTPUT ${_outdir} COMMAND mkdir ${_outdir} )
	
	_PCH_GET_COMPILE_FLAGS(_compile_FLAGS)
	
	ADD_CUSTOM_COMMAND(OUTPUT ${CMAKE_CURRENT_BINARY_DIR}/${_name} COMMAND ${CMAKE_COMMAND} -E copy  ${_input} ${CMAKE_CURRENT_BINARY_DIR}/${_name} )
	
	_PCH_GET_COMPILE_COMMAND(_command  ${_input} ${_output} )
	
	ADD_CUSTOM_COMMAND(OUTPUT ${_output} COMMAND ${_command} DEPENDS ${_input} ${_outdir} ${CMAKE_CURRENT_BINARY_DIR}/${_name} ${_targetName}_pchdep )
	
	ADD_CUSTOM_TARGET(${_targetName} DEPENDS ${_output} )
	
	ADD_DEPENDENCIES(${_targetName} ${_targetName}_gch )
	
	_PCH_GET_TARGET_COMPILE_FLAGS(_target_cflags ${_name} ${_output})
	
	SET_TARGET_PROPERTIES(${_targetName} PROPERTIES COMPILE_FLAGS ${_target_cflags}  )
ENDMACRO(ADD_PRECOMPILED_HEADER)
