/*  This file is part of the KDE project
    Copyright (C) 2007 Matthias Kretz <kretz@kde.org>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License version 2 as published by the Free Software Foundation.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02110-1301, USA.

*/

#ifndef PHONON_MIXERINTEGERCONTROL_H
#define PHONON_MIXERINTEGERCONTROL_H

#include "phonon_mixer_export.h"
#include "mixercontrol.h"

namespace Phonon
{
namespace Mixer
{

class MixerIntegerControlPrivate;
class PHONON_MIXER_EXPORT MixerIntegerControl : public MixerControl
{
    friend class Mixer;
    friend class MixerSharedData;
    friend MixerIntegerControl mixercontrol_cast<MixerIntegerControl>(const MixerControl &mc);
    friend MixerIntegerControl qobject_cast<MixerIntegerControl>(QObject *o);
    Q_DECLARE_PRIVATE(MixerIntegerControl)
    public:
        enum Category {
            LeftChannel,
            RightChannel,
            MonoChannel
        };
        MixerIntegerControl(const MixerIntegerControl &);
        MixerIntegerControl &operator=(const MixerIntegerControl &);

        int minimumValue() const;
        int maximumValue() const;
        int singleStep() const;
        int pageStep() const;
        int value() const;
        void setValue(int);

        Category category() const;

    protected:
        MixerIntegerControl(MixerIntegerControlPrivate &);
};

template<> PHONON_MIXER_EXPORT MixerIntegerControl mixercontrol_cast<MixerIntegerControl>(const MixerControl &mc);

} // namespace Mixer
} // namespace Phonon

template<> PHONON_MIXER_EXPORT Phonon::Mixer::MixerIntegerControl qobject_cast<Phonon::Mixer::MixerIntegerControl>(QObject *o);

#endif // PHONON_MIXERINTEGERCONTROL_H
